/*
 * sisctrl - SiS Display Control Panel
 * for the SiS XFree86 driver
 *
 * (C) 2003 Thomas Winischhofer <thomas@winischhofer.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef _SISCTRL_H_
#define _SISCTRL_H_

#include <config.h>

#define SISVERSION PACKAGE_VERSION

#if defined(HAVE_SYS_SHM_H) && defined(HAVE_XV)
#define USEXV
#endif

/* I don't check for HAVE_'s here, because the program
 * won't compile anyway if any of the headers is missing
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <signal.h>
#include <fcntl.h>
#ifdef HAVE_MATH_H
#include <math.h>
#endif
#include <sys/stat.h>
#include <sys/time.h>
#include <time.h>
#include <X11/Xatom.h>
#include <X11/extensions/xf86vmode.h>
#include <X11/extensions/xf86vmstr.h>
#ifdef USEXV
#include <X11/extensions/XShm.h>   /* Include BEFORE Xvlib.h! */
#endif
#include <X11/extensions/Xvlib.h>
#ifdef USEXV
#include <sys/shm.h>
#endif

#ifdef HAVE_RANDR
#include <X11/Xproto.h>
#include <X11/extensions/Xrandr.h>
#include <X11/extensions/Xrender.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include "../gfx/mode.xpm"
#include "../gfx/crt1.xpm"
#include "../gfx/crt2.xpm"
#include "../gfx/gamma.xpm"
#include "../gfx/tvsettings.xpm"
#include "../gfx/video.xpm"
#include "../gfx/config.xpm"
#include "../gfx/about.xpm"

#include "../gfx/lcd.xpm"
#include "../gfx/lcda.xpm"
#include "../gfx/tvpal.xpm"
#include "../gfx/tvpalm.xpm"
#include "../gfx/tvpaln.xpm"
#include "../gfx/tvntsc.xpm"
#include "../gfx/tvhivi.xpm"
#include "../gfx/tv525i.xpm"
#include "../gfx/tv525p.xpm"
#include "../gfx/tv750p.xpm"
#include "../gfx/tv1080i.xpm"
#include "../gfx/vga2.xpm"

#include "../gfx/crt1off.xpm"
#include "../gfx/crt1on.xpm"
#include "../gfx/crt1lcda.xpm"

#include "../gfx/crt2lcd.xpm"
#include "../gfx/crt2tv.xpm"
#include "../gfx/crt2svideo.xpm"
#include "../gfx/crt2cvbs.xpm"
#include "../gfx/crt2ypbpr.xpm"
#include "../gfx/crt2vga.xpm"

#include "../gfx/gammaicon.xpm"

#include "../gfx/tvleft.xpm"
#include "../gfx/tvright.xpm"
#include "../gfx/tvup.xpm"
#include "../gfx/tvdown.xpm"
#include "../gfx/tvxp.xpm"
#include "../gfx/tvxm.xpm"
#include "../gfx/tvyp.xpm"
#include "../gfx/tvym.xpm"

#include "../gfx/upperleft.xpm"
#include "../gfx/lowerright.xpm"
#include "../gfx/sisctrl.xpm"

#include "../gfx/ok.xpm"
#include "../gfx/apply.xpm"
#include "../gfx/close.xpm"
#include "../gfx/undo.xpm"

#include "../gfx/sisctrl_icon16.xpm"
#include "../gfx/sisctrl_icon32.xpm"
#include "../gfx/sisctrl_icon48.xpm"
#include "../gfx/sisctrl_icon64.xpm"

#ifdef USEXV
#include "../gfx/xvicon.h"
#endif

#define LOCKFILENAME "/tmp/sisCtrl.lock"

/* Maximum amount of modes this tool
 * can handle in the display mode tab
 */
#define MYMAXMODES 64

/* Size of gamma curve gfx (64 + 2 for border) */
#define GACW 66
#define GACH 66

/* Size of Xv icon image for video page */
#define XVWIDTH  150
#define XVHEIGHT 125

/* Minimum vidmodextension version required */
#define MINMAJOR 2
#define MINMINOR 0

/* Minimum randr extension version required */
#define RRMINMAJOR 1
#define RRMINMINOR 1

/* Why aren't these defined in xf86vmode*.h ? */
#define V_INTERLACE     0x010
#define V_DBLSCAN       0x020

/* VBFlags
 * This is taken from the current X driver and needs to be updated
 * if these defs change
 */
#define CRT2_DEFAULT            0x00000001
#define CRT2_LCD                0x00000002
#define CRT2_TV                 0x00000004
#define CRT2_VGA                0x00000008
#define TV_NTSC                 0x00000010
#define TV_PAL                  0x00000020
#define TV_HIVISION             0x00000040
#define TV_YPBPR                0x00000080
#define TV_AVIDEO               0x00000100
#define TV_SVIDEO               0x00000200
#define TV_SCART                0x00000400
#define VB_CONEXANT		0x00000800
#define TV_PALM                 0x00001000
#define TV_PALN                 0x00002000
#define TV_NTSCJ		0x00001000
#define VB_302ELV		0x00004000
#define TV_CHSCART              0x00008000
#define TV_CHYPBPR525I          0x00010000
#define CRT1_VGA		0x00000000
#define CRT1_LCDA		0x00020000
#define VGA2_CONNECTED          0x00040000
#define DISPTYPE_CRT1		0x00080000  	/* CRT1 connected and used */
#define VB_301                  0x00100000	/* Video bridge type */
#define VB_301B                 0x00200000
#define VB_302B                 0x00400000
#define VB_30xBDH		0x00800000      /* 30xB DH version (w/o LCD support) */
#define VB_LVDS                 0x01000000
#define VB_CHRONTEL             0x02000000
#define VB_301LV                0x04000000
#define VB_302LV                0x08000000
#define VB_301C			0x10000000
#define SINGLE_MODE             0x20000000   	/* CRT1 or CRT2; determined by DISPTYPE_CRTx */
#define MIRROR_MODE		0x40000000   	/* CRT1 + CRT2 identical (mirror mode) */
#define DUALVIEW_MODE		0x80000000   	/* CRT1 + CRT2 independent (dual head mode) */

/* Aliases: */
#define CRT2_ENABLE		(CRT2_LCD | CRT2_TV | CRT2_VGA)
#define TV_STANDARD             (TV_NTSC | TV_PAL | TV_PALM | TV_PALN | TV_NTSCJ)
#define TV_INTERFACE            (TV_AVIDEO|TV_SVIDEO|TV_SCART|TV_HIVISION|TV_YPBPR|TV_CHSCART|TV_CHYPBPR525I)

/* Only if TV_YPBPR is set: */
#define TV_YPBPR525I		TV_NTSC
#define TV_YPBPR525P		TV_PAL
#define TV_YPBPR750P		TV_PALM
#define TV_YPBPR1080I		TV_PALN
#define TV_YPBPRALL 		(TV_YPBPR525I | TV_YPBPR525P | TV_YPBPR750P | TV_YPBPR1080I)

#define TV_YPBPR43LB		TV_CHSCART
#define TV_YPBPR43		TV_CHYPBPR525I
#define TV_YPBPR169 		(TV_CHSCART | TV_CHYPBPR525I)
#define TV_YPBPRAR              (TV_CHSCART | TV_CHYPBPR525I)

#define VB_SISBRIDGE            (VB_301|VB_301B|VB_301C|VB_302B|VB_301LV|VB_302LV|VB_302ELV)
#define VB_SISTVBRIDGE          (VB_301|VB_301B|VB_301C|VB_302B|VB_301LV|VB_302LV)
#define VB_VIDEOBRIDGE		(VB_SISBRIDGE | VB_LVDS | VB_CHRONTEL | VB_CONEXANT)

#define DISPTYPE_DISP2		CRT2_ENABLE
#define DISPTYPE_DISP1		DISPTYPE_CRT1
#define VB_DISPMODE_SINGLE	SINGLE_MODE  	/* alias */
#define VB_DISPMODE_MIRROR	MIRROR_MODE  	/* alias */
#define VB_DISPMODE_DUAL	DUALVIEW_MODE 	/* alias */
#define DISPLAY_MODE            (SINGLE_MODE | MIRROR_MODE | DUALVIEW_MODE)

#define BMASK (CRT2_ENABLE | TV_STANDARD | TV_INTERFACE | CRT1_LCDA)

/* SiS Direct Xv-API */
#define SiS_SD_IS300SERIES  	0x00000001
#define SiS_SD_IS315SERIES  	0x00000002
#define SiS_SD_IS330SERIES  	0x00000004
#define SiS_SD_SUPPORTPALMN 	0x00000008   /* tv chip supports pal-m, pal-n */
#define SiS_SD_SUPPORT2OVL  	0x00000010   /* set = 2 overlays, 1 = support SWITCHCRT xv prop */
#define SiS_SD_SUPPORTTVPOS 	0x00000020   /* supports changing tv position */
#define SiS_SD_ISDUALHEAD   	0x00000040   /* Driver is in dual head mode */
#define SiS_SD_ISMERGEDFB   	0x00000080   /* Driver is in merged fb mode */
#define SiS_SD_ISDHSECONDHEAD   0x00000100   /* Dual head: This is CRT1 (=second head) */
#define SiS_SD_ISDHXINERAMA   	0x00000200   /* Dual head: We are running Xinerama */
#define SiS_SD_VBHASSCART     	0x00000400   /* videobridge has SCART instead of VGA2 */
#define SiS_SD_ISDEPTH8       	0x00000800   /* Depth is 8, no independent gamma correction */
#define SiS_SD_SUPPORTSOVER     0x00001000   /* Support for Chrontel Super Overscan */
#define SiS_SD_ENABLED        	0x00002000   /* sisctrl is enabled (by option) */
#define SiS_SD_PSEUDOXINERAMA 	0x00004000   /* pseudo xinerama is active */
#define SiS_SD_SUPPORTLCDA    	0x00008000   /* Support LCD Channel A ("LCD via CRT1") */
#define SiS_SD_SUPPORTNTSCJ     0x00010000   /* tv chip supports ntsc-j */
#define SiS_SD_ADDLSUPFLAG      0x00020000   /* 1 = the following flags are valid: */
#define SiS_SD_SUPPORTVGA2	0x00040000   /* CRT2=VGA supported */
#define SiS_SD_SUPPORTSCART     0x00080000   /* CRT2=SCART supported */
#define SiS_SD_SUPPORTOVERSCAN  0x00100000   /* Overscan flag supported */
#define SiS_SD_SUPPORTXVGAMMA1  0x00200000   /* Supports Gamma correction for Xv on CRT1 */
#define SiS_SD_SUPPORTTV  	0x00400000   /* CRT2=TV supported */
#define SiS_SD_SUPPORTYPBPR     0x00800000   /* CRT2=YPbPr (525i, 525p, 750p) is supported */
#define SiS_SD_SUPPORTHIVISION  0x01000000   /* HiVision (1080i) is supported */
#define SiS_SD_SUPPORTYPBPRAR   0x02000000   /* YPbPr aspect ratio is supported */
#define SiS_SD_SUPPORTSCALE     0x04000000   /* Scaling of LCD panel supported */
#define SiS_SD_SUPPORTCENTER    0x08000000   /* If scaling supported: Centering of screen supported (TMDS only) */

#define SIS_DIRECTKEY       	0x3145792

/* ChipFlags */
/* Use only lower 16 bit for chip id! */
#define SiSCF_LARGEOVERLAY 0x0001
#define SiSCF_Is651        0x0002
#define SiSCF_IsM650       0x0004
#define SiSCF_IsM652       0x0008
#define SiSCF_IsM653       0x0010
#define SiSCF_Is652        0x0020
#define SiSCF_IsM661       0x0100 /* M661FX */
#define SiSCF_IsM741       0x0200
#define SiSCF_IsM760       0x0400
#define SiSCF_IsM661M      0x0800 /* M661MX */

/* Check mode for CRT2 */
#define SiS_CF2_LCD          0x01
#define SiS_CF2_TV           0x02
#define SiS_CF2_VGA2         0x04
#define SiS_CF2_TVPAL        0x08
#define SiS_CF2_TVNTSC       0x10  /* + NTSC-J */
#define SiS_CF2_TVPALM       0x20
#define SiS_CF2_TVPALN       0x40
#define SiS_CF2_CRT1LCDA     0x80
#define SiS_CF2_TYPEMASK     (SiS_CF2_LCD | SiS_CF2_TV | SiS_CF2_VGA2 | SiS_CF2_CRT1LCDA)
#define SiS_CF2_TVSPECIAL    (SiS_CF2_LCD | SiS_CF2_TV)
#define SiS_CF2_TVSPECMASK   (SiS_CF2_TVPAL | SiS_CF2_TVNTSC | SiS_CF2_TVPALM | SiS_CF2_TVPALN)
#define SiS_CF2_TVHIVISION   SiS_CF2_TVPAL
#define SiS_CF2_TVYPBPR525I  SiS_CF2_TVNTSC
#define SiS_CF2_TVYPBPR525P  (SiS_CF2_TVPAL | SiS_CF2_TVNTSC)
#define SiS_CF2_TVYPBPR750P  SiS_CF2_TVPALM
#define SiS_CF2_TVYPBPR1080I (SiS_CF2_TVPALM | SiS_CF2_TVPAL)

#define SiS_CRT2_LCD	    0x0001
#define SiS_CRT2_TV	    0x0002
#define SiS_CRT2_TVPAL	    0x0004
#define SiS_CRT2_TVNTSC	    0x0008
#define SiS_CRT2_TVPALM	    0x0010
#define SiS_CRT2_TVPALN	    0x0020
#define SiS_CRT2_CRT1LCDA   0x0040
#define SiS_CRT2_HIVISION   0x0080
#define SiS_CRT2_YPBPR525I  0x0100
#define SiS_CRT2_YPBPR525P  0x0200
#define SiS_CRT2_YPBPR750P  0x0400
#define SiS_CRT2_YPBPR1080I 0x0800
#define SiS_CRT2_VGA2       0x1000

/* Menu index for CRT2 menu */
#define MI_TV_LCD   0
#define MI_TV_SVHS  1
#define MI_TV_CVBS  2
#define MI_TV_CVSV  3
#define MI_TV_TV    4
#define MI_TV_HIVI  5
#define MI_TV_525I  6
#define MI_TV_525P  7
#define MI_TV_750P  8
#define MI_TV_1080I 9
#define MI_TV_VGA   10
#define MI_TV_SCART 11
#define MI_TV_OFF   12

typedef enum _SIS_CHIP_TYPE {
    SIS_VGALegacy = 0,
    SIS_530,
    SIS_OLD,
    SIS_300,
    SIS_630,
    SIS_730,
    SIS_540,
    SIS_315H,   /* SiS 310 */
    SIS_315,
    SIS_315PRO, /* SiS 325 */
    SIS_550,
    SIS_650,
    SIS_740,
    SIS_330,
    SIS_661,
    SIS_741,
    SIS_660,
    SIS_760,
    MAX_SIS_CHIP
} SIS_CHIP_TYPE;

char xverrorstring[] = "Failed to set/get Xv property\n";

typedef struct  {
	Display *dpy;
	int myscreen;

	int lockfile;
	char filenamebuffer[64];

#ifdef USEXV
	Window xvwin;
	GC xvcontext;
	XvImage *myxvimage;
	XShmSegmentInfo xvshminfo;
	Bool xvattached, xvshown, xvraised, xvinit, noxvdemo, xvwasbusy;
	int xvx, xvy, xvwidth, xvheight;
#endif

	XF86VidModeModeInfo **vmmodelines;
	int vmmodecount;
	XF86VidModeGamma currentgamma, newgamma;

	char *modetextptr[MYMAXMODES];
	unsigned short modecheckfield[MYMAXMODES];
	int moderate[MYMAXMODES];
	char *sizetextptr[MYMAXMODES+2];

	Bool haverandr;
#ifdef HAVE_RANDR
	Window	root;
      	XRRScreenSize *rrsizes;
  	XRRScreenConfiguration *sconf;
  	int	rrnsize, rrsize, rrrate, rrnrate, rrrot;
  	short	*rrrates;
	SizeID	rrcurrent_size, rrsize_backup;
  	short   rrcurrent_rate;
	Rotation rrcurrent_rotation, rrrotation_backup;
	Bool    rrcheck_backup;
	char    leaveunchangedtext[128];
#endif

	char *attr_name;
	int xv_port_nr;
	Atom atom_qvf, atom_usd, atom_qvv, atom_qdd, atom_tvx, atom_tvy, atom_svf;
	Atom atom_taf, atom_tsa, atom_tee, atom_tco, atom_tte, atom_ccc, atom_gsf;
	Atom atom_tcf, atom_tlf, atom_cmd, atom_cmdr,atom_ct1, atom_con, atom_ghi;
	Atom atom_bri, atom_hue, atom_sat, atom_swc, atom_sga, atom_gdv, atom_ovr;
	Atom atom_gbi, atom_txs, atom_tys, atom_coc, atom_cof, atom_cfi, atom_yfi;
	Atom atom_gss, atom_brr, atom_brg, atom_brb, atom_pbr, atom_pbg, atom_pbb;
	Atom atom_vgr, atom_vgg, atom_vgb, atom_col, atom_pmd;

	unsigned long currentvbflags, detecteddevices, newvbflags, backupvbflags, busid;
	unsigned long sdflags, gammaflags, backupgammaflags, hwinfo, driverversion;
	int CRT1isoff, CRT1isoffbackup, currentmodeindex, backupmodeindex;
	int choverscanstatus, backupoverscan, lcdscaler, backuplcdscaler;
	double backup_gred, backup_ggreen, backup_gblue;
	double gamma_max[3], gamma_prescale[3], gamma_max_b[3], gamma_pre_b[3];
	unsigned short *ramp[3];
	float framp;
	int nramp;
	float xvgammared, xvgammagreen, xvgammablue;
	unsigned long colorkey;

	Bool HaveGss;
	Bool HaveBri;

	Bool guimode;
	int givencrt2index, giventvstdindex, giventvosindex, giventvasindex;

	int tp_type;

	Bool dorandr;

	/* GUI stuff */
	GtkWidget *window, *errordialog;
	GtkWidget *table, *tv_table;
	GtkWidget *notebook;
	Bool nbscroll;
	int notebook_page;
	GtkWidget *dm_frame, *c1_frame, *om_frame, *ga_frame;
	GtkWidget *tv_frame, *tp_frame, *vi_frame, *ab_frame;
	GtkWidget *dm_table, *dm_menu, *dm_omenu, *dm_size_label;
	GtkWidget *dm_list, *dm_list_window;
	GtkListStore *dm_liststore;
	GtkTreeIter dm_iter[MYMAXMODES];
	GtkCellRenderer *dm_renderer;
	GtkTreeViewColumn *dm_column;
	GtkTreeSelection *dm_select;
	GtkWidget *dm_menuitem[MYMAXMODES+2];
	GtkWidget *c1_table, *c1_menu, *c1_omenu;
	GtkWidget *c1_menuitemon, *c1_menuitemoff, *c1_menuitemlcda;
	GtkWidget *c1_label_nodh, *c1_label_onoff, *c1_label_lcds;
	GtkWidget *c1_imageon, *c1_imageoff, *c1_imagelcda;
	GdkPixmap *c1_iconon, *c1_iconoff, *c1_iconlcda;
	GdkBitmap *c1_maskon, *c1_maskoff, *c1_masklcda;
	GtkWidget *c1_menuls, *c1_omenuls, *c1_menulcc, *c1_omenulcc;
	GtkWidget *c1_menuitemlsd, *c1_menuitemlso, *c1_menuitemlsf;
	GtkWidget *c1_menuitemlccd, *c1_menuitemlccp, *c1_menuitemlccc;
	GtkWidget *om_table, *om_menu, *om_omenu;
	GtkWidget *om_menut, *om_omenut;
	GtkWidget *om_menuitemlcd, *om_menuitemtv;
	GtkWidget *om_menuitemsvhs, *om_menuitemcvbs, *om_menuitemcvsv;
	GtkWidget *om_menuitemvga2, *om_menuitemscart, *om_menuitemoff;
	GtkWidget *om_menuitemhivi, *om_menuitemy525i, *om_menuitemy525p;
	GtkWidget *om_menuitemy750p, *om_menuitemy1080i;
	GtkWidget *om_menuitempal, *om_menuitemntsc;
	GtkWidget *om_menuitempalm, *om_menuitempaln, *om_menuitemntscj;
	GtkWidget *om_menuas, *om_omenuas, *om_label_tvas;
	GtkWidget *om_menuitem43, *om_menuitem43lb, *om_menuitem169;
	GtkWidget *om_menuo, *om_omenuo;
	GtkWidget *om_menuitembios, *om_menuitemunder;
	GtkWidget *om_menuitemover, *om_menuitemsover;
	GtkWidget *om_menuls, *om_omenuls, *om_menulcc, *om_omenulcc, *om_label_lcds;
	GtkWidget *om_menuitemlsd, *om_menuitemlso, *om_menuitemlsf;
	GtkWidget *om_menuitemlccd, *om_menuitemlccp, *om_menuitemlccc;
	GtkWidget *om_lcd, *om_tv, *om_tvs, *om_tvc, *om_vga, *om_ypbpr;
	GdkPixmap *om_ilcd, *om_itv, *om_itvs, *om_itvc, *om_ivga, *om_iypbpr;
	GdkBitmap *om_mlcd, *om_mtv, *om_mtvs, *om_mtvc, *om_mvga, *om_mypbpr;
	GtkWidget *om_check, *om_label_nodh, *om_label_type, *om_label_tvtype;
	GtkWidget *ga_table, *ga_check1, *ga_check2, *ga_spin1, *ga_spin2, *ga_spin3;
	GtkWidget *ga_spin4, *ga_spin5, *ga_spin6, *ga_spin7, *ga_spin8, *ga_spin9;
	GtkAdjustment *ga_spin1_adj,*ga_spin2_adj,*ga_spin3_adj;
	GtkAdjustment *ga_spin4_adj,*ga_spin5_adj,*ga_spin6_adj;
	GtkAdjustment *ga_spin7_adj,*ga_spin8_adj,*ga_spin9_adj;
	GtkWidget *ga_drawable, *ga_subtable, *ga_plabel1, *ga_plabel2, *ga_hbox, *ga_vbox;
	GtkWidget *ga_label_enable, *ga_label_red, *ga_label_green, *ga_label_blue;
	GtkWidget *ga_label_red1, *ga_label_green1, *ga_label_blue1;
	GtkWidget *ga_label_red2, *ga_label_green2, *ga_label_blue2;
	GtkWidget *ga_label_gamma, *ga_label_bri, *ga_label_prebri, *ga_image;
	GdkPixmap *ga_icon;
	GdkBitmap *ga_mask;
	GtkWidget *tp_table, *tp_menu, *tp_omenu, *tp_box, *tp_menuy, *tp_omenuy;
	GtkWidget *tp_label_notv, *tv_label_notv;
	GtkWidget *tp_label_af, *tp_label_sa, *tp_label_ee, *tp_label_coc, *tp_label_cof;
	GtkWidget *tp_label_co, *tp_label_te, *tp_check_col, *tp_check_cfi, *tp_label_yfi;
	GtkWidget *tp_label_cf, *tp_label_lf;
	GtkWidget *tp_range_sa, *tp_range_ee, *tp_range_coc, *tp_range_cof;
	GtkWidget *tp_range_co, *tp_range_te;
	GtkWidget *tp_range_cf, *tp_range_lf;
	GtkWidget *tp_menuitem1, *tp_menuitem2;
	GtkWidget *tp_menuitem3, *tp_menuitem4, *tp_menuitem5;
	GtkWidget *tp_menuitemy1, *tp_menuitemy2, *tp_menuitemy3;
	GtkWidget *tp_menuitemy4, *tp_menuitemy5, *tp_menuitemy6;
	GtkWidget *tp_menuitemy7, *tp_menuitemy8, *tp_menuitemy9;
	GtkWidget *tv_button_le, *tv_button_ri, *tv_button_dn, *tv_button_up;
	GtkWidget *tv_button_de;
	GtkWidget *tv_button_xm, *tv_button_xp, *tv_button_yp, *tv_button_ym;
	GtkWidget *tv_arrow_ri, *tv_arrow_le, *tv_arrow_dn, *tv_arrow_up;
	GtkWidget *tv_arrow_xp, *tv_arrow_xm, *tv_arrow_yp, *tv_arrow_ym;
	GtkWidget *vi_table, *vi_table2, *vi_ga_table, *vi_layout;
	GtkWidget *vi_label_co, *vi_label_br, *vi_label_sa, *vi_label_hu, *vi_label_sw;
	GtkWidget *vi_range_co, *vi_range_br, *vi_range_sa, *vi_range_hu;
	GtkWidget *vi_label_red, *vi_label_green, *vi_label_blue;
	GtkWidget *vi_check_gamma, *vi_spin1, *vi_spin2, *vi_spin3;
	GtkAdjustment *vi_spin1_adj, *vi_spin2_adj, *vi_spin3_adj;
	GtkWidget *vi_radio_crt1, *vi_radio_crt2;
	GSList    *vi_group;
	GtkWidget *cf_text_window, *cf_tview, *cf_frame, *cf_label;
	GtkTextBuffer *cf_buffer;
	GtkWidget *ab_table, *ab_label_ab1, *ab_label_ab2, *ab_layout;
	GtkWidget *ab_label_ab3, *ab_label_ab4, *ab_label_ab5;
	GdkPixmap *ab_icon, *ab_iconlr, *ab_iconul;
        GdkBitmap *ab_mask, *ab_masklr, *ab_maskul;
	GtkWidget *dm_label, *c1_label, *om_label, *ga_label;
	GtkWidget *tv_label, *tp_label, *vi_label, *ab_label;
	GtkWidget *button1, *button2, *button3, *button4;
	GtkWidget *button1box, *button2box, *button3box, *button4box;
	GtkTextTag *black, *grey, *red;

	GdkColor  ab_bgcolor;

	GdkPixbuf *pixbuf_lcd, *pixbuf_lcda, *pixbuf_tvpal, *pixbuf_tvpalm;
        GdkPixbuf *pixbuf_tvpaln, *pixbuf_tvntsc, *pixbuf_tvhivi, *pixbuf_tv525i;
        GdkPixbuf *pixbuf_tv525p, *pixbuf_tv750p, *pixbuf_tv1080i, *pixbuf_vga2;

	guchar rgbbuf[GACW * GACH * 3];
} sc_globals;

sc_globals sc;

/* forward declarations */
void update_gamma_gfx(void);
static void setxvgammastatus(void);
static void setxvgammasensitive(void);
#ifdef USEXV
static void destroyxvimage(void);
#endif

#endif
